//-------------------------------------------------------------------------
// Filename:      ossmidi.hh
// Version:       $Id: ossmidi.hh,v 1.1.2.2 1999/06/09 10:14:03 erik Exp $
// Copyright:     Copyright (C) 1999, Erik Mouw (J.A.K.Mouw@its.tudelft.nl)
// Author:        Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
// Description:   Play MIDI events on an OSS MIDI interface
// Created at:    Mon May 24 17:44:06 1999
// Modified by:   Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
// Modified at:   Wed Jun  9 12:11:53 1999
//-------------------------------------------------------------------------
// 
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#ident "$Id: ossmidi.hh,v 1.1.2.2 1999/06/09 10:14:03 erik Exp $"

#ifndef musica_ossmidi_hh
#define musica_ossmidi_hh




#include "mididevice.hh"




class ossSynth : virtual public midiDevice
{
protected:
  char *device;
  int fd;
  int deviceNo;

public:
  ossSynth(const char *theName);
  virtual ~ossSynth(void);

  void setDevice(char *filename, int number);

  // fill in virtual functions
  virtual bool open(void);
  virtual bool close(void);

  virtual void noteOn(int channel, int key, int velocity);
  virtual void noteOff(int channel, int key, int velocity);
  
  virtual void keyPressure(int channel, int key, int pressure);
  virtual void channelPressure(int channel, int pressure);
  
  virtual void pitchBend(int channel, int fine, int coarse);

  virtual void controlChange(int channel, int controller, int value);

  virtual void programChange(int channel, int program);

  virtual void allNoteOff(void);

  virtual void systemReset(void);
};




class ossSequencer : virtual public midiDevice
{
private:
  void seqbuf_dump();

protected:
  char *device;
  int fd;
  int deviceNo;

public:
  ossSequencer(const char *theName);
  virtual ~ossSequencer(void);

  void setDevice(char *filename, int number);

  // fill in virtual functions
  virtual bool open(void);
  virtual bool close(void);

  virtual void noteOn(int channel, int key, int velocity);
  virtual void noteOff(int channel, int key, int velocity);
  
  virtual void keyPressure(int channel, int key, int pressure);
  virtual void channelPressure(int channel, int pressure);
  
  virtual void pitchBend(int channel, int fine, int coarse);

  virtual void controlChange(int channel, int controller, int value);

  virtual void programChange(int channel, int program);

  virtual void allNoteOff(void);

  virtual void systemReset(void);
};




class ossMidi: virtual public midiDevice
{
protected:
  int deviceIndex;
  char *device;
  int fd;

public:
  ossMidi(const char *theName);
  virtual ~ossMidi(void);

  void setDevice(char *filename);

  // fill in virtual functions
  virtual bool open(void);
  virtual bool close(void);

  virtual void noteOn(int channel, int key, int velocity);
  virtual void noteOff(int channel, int key, int velocity);
  
  virtual void keyPressure(int channel, int key, int pressure);
  virtual void channelPressure(int channel, int pressure);
  
  virtual void pitchBend(int channel, int fine, int coarse);

  virtual void controlChange(int channel, int controller, int value);

  virtual void programChange(int channel, int program);

  virtual void allNoteOff(void);

  virtual void systemReset(void);
};


#endif
