/*-------------------------------------------------------------------------
 * Filename:      memcmp.c
 * Version:       $Id: memcmp.c,v 1.2 1999/05/02 20:26:27 erik Exp $
 * Copyright:     Copyright (C) 1999, Erik Mouw
 * Author:        Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Description:   memcmp() replacement function for brain damaged OSes
 * Created at:    Sat May  1 12:51:04 1999
 * Modified by:   Erik Mouw <J.A.K.Mouw@its.tudelft.nl>
 * Modified at:   Sun May  2 16:49:11 1999
 *-----------------------------------------------------------------------*/
/*
 * This will be compiled if your system misses a clean memcmp() function.
 *
 *
 * memcmp.c, Copyright (C) 1999, Erik Mouw
 *
 * Email:
 *   J.A.K.Mouw@its.tudelft.nl
 *
 * Snail mail:
 *   J.A.K. Mouw
 *   Information and Communication Theory Group
 *   Department of Electrical Engineering
 *   Faculty of Information Technology and Systems
 *   Delft University of Technology
 *   P.O. Box 5031
 *   2600 GA  Delft
 *   The Netherlands
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software (the "Software"), to deal in the
 * Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute,
 * sublicense, and/or sell copies of the Software, and to permit
 * persons to whom the Software is furnished to do so, subject to the
 * following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 * Except as contained in this notice, the name of Erik Mouw shall not
 * be used in advertising or otherwise to promote the sale, use or
 * other dealings in this Software without prior written authorization
 * from Erik Mouw.
 *
 *
 * memcmp - compare two chunks of memory
 *
 * The memcmp() function compares the first n bytes of the memory
 * chunks s1 and s2. It returns an integer less than, equal to, or
 * greater than zero if s1 is found, respectively, to be less than, to
 * match, or be greater than s2.
 * 
 */

#ident "$Id: memcmp.c,v 1.2 1999/05/02 20:26:27 erik Exp $"

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif

#if STDC_HEADERS
# include <stdio.h>
# include <stdlib.h>
#else
# ifdef HAVE_MALLOC_H
#  include <malloc.h>
# else
#  ifndef NULL
#   define NULL ((void *)0)
#  endif
# endif
#endif








#ifdef __cplusplus
extern "C" int memcmp(const void *s1, const void *s2, size_t n)
#else
int memcmp(const void *s1, const void *s2, size_t n)
#endif
{
  const unsigned char *i1;
  const unsigned char *i2;

  /* check for equal pointers or zero length comparisons */
  if((s1 == s2) || (n == 0))
    return(0);

  /* s1 is NULL, so it must be smaller than s2 */
  if(s1 == NULL)
    return(-1);

  /* s2 is NULL, so it must be smaller than s1 */
  if(s2 == NULL)
    return(1);

  /* compare the two chunks of memory */
  for(i1 = s1, i2 = s2; n > 0; i1++, i2++, n--)
    {
      if(*i1 != *i2)
	return((*i1 < * i2) ? -1 : 1);
    }

  /* they were equal */
  return(0);
}
